require 'spec_helper'

describe Diff do

  def fixture(name)
    File.read(File.join(File.dirname(__FILE__), "../fixtures/reports/#{name}.yaml"))
  end

  before :each do
    Summarizer.stubs(:delay).returns(Summarizer)
  end

  describe '.collate_properties_by_resource' do
    describe 'results' do
      before :each do
        baseline = Report.create_from_yaml(fixture 'baseline')
        report   = Report.create_from_yaml(fixture 'inspect')
        @node_baseline = NodeBaseline.create_from_report!(baseline, report.time)
        Summarizer.build_baseline_summary(report.id, @node_baseline)
      end

      subject do
        Diff.collate_properties_by_resource(@node_baseline)
      end

      it 'should contain resources present in both the baseline and inspection' do
        subject.map(&:first).should include(['File', '/etc/sudoers'])
      end

      it 'should contain resources identical in both the baseline and inspection' do
        subject.map(&:first).should include(['File', '/identical'])
      end

      it 'should contain resources present only in the baseline' do
        subject.map(&:first).should include(['File', '/baseline'])
      end

      it 'should contain resources present only in the inspection' do
        subject.map(&:first).should include(['File', '/inspect'])
      end

      it 'should contain properties present in both the baseline and inspection' do
        res = subject.find { |e| e.first == ['File', '/etc/sudoers'] }
        res.should_not be_nil
        res.last.should include(["mode", {:inspected => "440", :baseline => "400"}])
      end

      it 'should contain properties identical in both the baseline and inspection' do
        res = subject.find { |e| e.first == ['File', '/etc/sudoers'] }
        res.should_not be_nil
        res.last.should include(["type", {:inspected => "file", :baseline => "file"}])
      end

      it 'should contain properties present only in the baseline' do
        res = subject.find { |e| e.first == ['File', '/etc/sudoers'] }
        res.should_not be_nil
        res.last.should include(["baseline_only", {:inspected => nil, :baseline => :baseline_only}])
      end

      it 'should contain properties present only in the inspection' do
        res = subject.find { |e| e.first == ['File', '/etc/sudoers'] }
        res.should_not be_nil
        res.last.should include(["inspect_only", {:inspected => :inspect_only, :baseline => nil}])
      end
    end

    it 'should provide the expected results' do
        baseline = Report.create_from_yaml(fixture 'baseline-node')
        report   = Report.create_from_yaml(fixture 'inspected-node')
        node_baseline = NodeBaseline.create_from_report!(baseline, report.time)
        Summarizer.build_baseline_summary(report.id, node_baseline)
        Diff.collate_properties_by_resource(node_baseline).should == [
          [['File', '/baseline'], [
            ['ensure', {
              :baseline => :file,
              :inspected => nil,
            }],
          ]],
          [['File', '/common'], [
            ['content', {
              :baseline => '{md5}338fd571d98a6847b305781add56703f',
              :inspected => '{md5}147c263ec80caa6852d3f843e36108d6',
            }],
            ['owner', {
              :baseline => "0",
              :inspected => "0",
            }],
          ]],
          [['File', '/inspected'], [
            ['ensure', {
              :baseline => nil,
              :inspected => :file,
            }],
          ]],
        ]
    end
  end

  describe ".node_against_baseline" do
    it "should delegate to node_diff with the appropriate node baseline" do
      baseline_report = Report.create_from_yaml(fixture 'baseline')
      report          = Report.create_from_yaml(fixture 'inspect')

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Diff.expects(:node_diff).with(node_baseline, 0, 100).returns :something
      Diff.node_against_baseline(report.node_id, report.time, 0, 100).should == :something
    end
  end

  describe ".node_diff" do
    it "should return the diff of the node's inspect report vs. its baseline, for the given date" do
      baseline_report = Report.create_from_yaml(fixture 'baseline')
      report          = Report.create_from_yaml(fixture 'inspect')

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Summarizer.build_baseline_summary(report.id, node_baseline)

      Diff.node_diff(node_baseline, 0, 100).should == [
        {
          :resource_name => "File[/baseline]",
          :resource_type => "File",
          :resource_title => "/baseline",
          :properties => {
            "ensure" => { :baseline => :file },
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/baseline]').id
        },
        {
          :resource_name => "File[/etc/hosts]",
          :resource_type => "File",
          :resource_title => "/etc/hosts",
          :properties => {
            "owner" => {
              :baseline  => "40",
              :inspected => "0"
            },
            "group" => {
              :baseline  => "50",
              :inspected => "0"
            },
            "mode" => {
              :baseline => "666",
              :inspected => "644"
            }
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/etc/hosts]').id
        },
        {
          :resource_name => "File[/etc/sudoers]",
          :resource_type => "File",
          :resource_title => "/etc/sudoers",
          :properties => {
            "inspect_only" => { :inspected=>:inspect_only },
            "baseline_only" => { :baseline=>:baseline_only },
            "mode" => {
              :baseline => "400",
              :inspected => "440"
            }
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/etc/sudoers]').id
        },
        {
          :resource_name => "File[/inspect]",
          :resource_type => "File",
          :resource_title => "/inspect",
          :properties => {
            "ensure" => { :inspected => :file },
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/inspect]').id
        },
      ]
    end

    it "should return an empty hash if there are no differences" do
      report = Report.create_from_yaml(fixture 'foo')
      node_baseline = NodeBaseline.create_from_report!(report, report.time)
      Summarizer.build_baseline_summary(report.id, node_baseline)
      Diff.node_diff(node_baseline, 0, 100).should == []
    end

    it "should omit corresponding values for resources not present in the report" do
      baseline_report = Report.create_from_yaml(fixture 'foo')

      report = Factory(:inspect_report, :host => baseline_report.host, :time => baseline_report.time + 1.hour)

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Summarizer.build_baseline_summary(report.id, node_baseline)

      Diff.node_diff(node_baseline, 0, 100).should == [
        {
          :resource_name => "File[/etc/hosts]",
          :resource_type => "File",
          :resource_title => "/etc/hosts",
          :properties => {
            "owner" => {:baseline => "0"},
            "group" => {:baseline => "0"},
            "mode" => {:baseline => "644"},
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/etc/hosts]').id
        },
        {
          :resource_name => "File[/etc/sudoers]",
          :resource_type => "File",
          :resource_title => "/etc/sudoers",
          :properties => {
            "owner" => {:baseline => "0"},
            "group" => {:baseline => "0"},
            "mode" => {:baseline => "440"},
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/etc/sudoers]').id
        },
        {
          :resource_name => "User[root]",
          :resource_type => "User",
          :resource_title => "root",
          :properties => {
            "home" => {:baseline => "/var/root"},
            "shell" => {:baseline => "/bin/sh"},
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('User[root]').id
        },
      ]
    end

    it "should omit corresponding values for resources not present in the baseline" do
      report = Report.create_from_yaml(fixture 'foo')
      baseline_report = Factory(:inspect_report, :host => report.host, :time => report.time + 1.hour)

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Summarizer.build_baseline_summary(report.id, node_baseline)

      Diff.node_diff(node_baseline, 0, 100).should == [
        {
          :resource_name => "File[/etc/hosts]",
          :resource_type => "File",
          :resource_title => "/etc/hosts",
          :properties => {
            "owner" => {:inspected => "0"},
            "group" => {:inspected => "0"},
            "mode" => {:inspected => "644"},
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/etc/hosts]').id
        },
        {
          :resource_name => "File[/etc/sudoers]",
          :resource_type => "File",
          :resource_title => "/etc/sudoers",
          :properties => {
            "owner" => {:inspected => "0"},
            "group" => {:inspected => "0"},
            "mode" => {:inspected => "440"},
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/etc/sudoers]').id
        },
        {
          :resource_name => "User[root]",
          :resource_type => "User",
          :resource_title => "root",
          :properties => {
            "home" => {:inspected => "/var/root"},
            "shell" => {:inspected => "/bin/sh"},
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('User[root]').id
        },
      ]
    end

    it "should filter results by resource name" do
      baseline_report = Report.create_from_yaml(fixture 'baseline')
      report          = Report.create_from_yaml(fixture 'inspect')

      node_baseline = NodeBaseline.create_from_report!(baseline_report, report.time)

      Summarizer.build_baseline_summary(report.id, node_baseline)

      Diff.node_diff(node_baseline, 0, 100, 'file', 'sudoers').should == [
        {
          :resource_name => "File[/etc/sudoers]",
          :resource_type => "File",
          :resource_title => "/etc/sudoers",
          :properties => {
            "inspect_only" => { :inspected=>:inspect_only },
            "baseline_only" => { :baseline=>:baseline_only },
            "mode" => {
              :baseline => "400",
              :inspected => "440"
            }
          },
          :accepted => nil,
          :can_change => true,
          :diff_id => BaselineDifference.find_by_resource_name('File[/etc/sudoers]').id
        }
      ]
    end
  end

  describe ".group_against_node" do
    it "should return a hash of the common outcome and the nodes with those outcomes" do
      foo_report = Report.create_from_yaml(fixture 'foo')
      bar_report = Report.create_from_yaml(fixture 'bar')
      baz_report = Report.create_from_yaml(fixture 'baz')

      group = NodeGroup.generate!
      group.nodes = [foo_report.node, bar_report.node, baz_report.node]
      group.save!

      baseline = NodeBaseline.create_from_report!(baz_report, baz_report.time)

      Summarizer.compare_group_against_node(group, baz_report.node_id)

      diff = Diff.group_against_node(group, baz_report.node, baz_report.time, 0, 5)

      diff.should == [
        [
          "File[/etc/hosts]", [
            {
              :properties => {
                :inspected => {
                  "group" => "1",
                  "mode" => "655",
                  "owner" => "0",
                },
                :baseline => [
                  {
                    "group" => "0",
                    "mode" => "644",
                    "owner" => "1",
                  },
                ],
              },
              :digest => "Dz11WOTDLc8YwzGN+3ts8w==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1,
            },
            {
              :properties => {
                :inspected => {
                  "owner" => "0",
                },
                :baseline => [
                  {
                    "owner" => "1",
                  },
                ],
              },
              :digest => "Pa1BjVFP4aRZxctihvTFjw==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1,
            },
          ],
        ],
        [
          "File[/etc/sudoers]", [
            {
              :properties => {
                :inspected => {
                  "owner" => "0",
                },
                :baseline => [
                  {
                    "owner" => "1",
                  },
                ],
              },
              :digest => "BdGCiBagW+SY+vZqKAvQdg==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 2,
            },
          ],
        ],
      ]
    end

    it "should omit resources with no differences" do
      foo_report = Report.create_from_yaml(fixture 'foo')

      group = NodeGroup.generate!
      group.nodes = [foo_report.node]
      group.save!

      baseline = NodeBaseline.create_from_report!(foo_report, foo_report.time)

      Summarizer.compare_group_against_node(group, foo_report.node_id)

      diff = Diff.group_against_node(group, foo_report.node, foo_report.time, 0, 5)

      diff.should == []
    end

    it "should paginate resources by name" do
      foo_report                = Report.create_from_yaml(fixture 'foo')
      foo_common_changes_report = Report.create_from_yaml(fixture 'foo_common_changes')
      baz_report                = Report.create_from_yaml(fixture 'baz')

      group = NodeGroup.generate!
      group.nodes = [foo_report.node, foo_common_changes_report.node, baz_report.node]
      group.save!

      baseline = NodeBaseline.create_from_report!(baz_report, baz_report.time)

      Summarizer.compare_group_against_node(group, baz_report.node_id)

      diff = Diff.group_against_node(group, baz_report.node, baz_report.time, 1, 1)
      diff.should == [
        [
          "File[/etc/sudoers]",
          [
            {
              :node_count => 2,
              :accepted => 'no-action',
              :can_change => true,
              :digest => [Digest::MD5.digest("group0mode440owner0")].pack('m').chomp,
              :baseline_count => 1,
              :properties => {
                :baseline => [{
                  "owner" => "1"
                }],
                :inspected => {
                  "owner" => "0"
                }
              }
            }
          ]
        ]
      ]
    end

    it "should use an accepted value of 'multiple' when nodes with the same change have inconsistent accept states" do
      foo_report = Report.create_from_yaml(fixture 'foo')
      bar_report = Report.create_from_yaml(fixture 'bar')
      baz_report = Report.create_from_yaml(fixture 'baz')

      group = NodeGroup.generate!
      group.nodes = [foo_report.node, bar_report.node, baz_report.node]
      group.save!

      baseline = NodeBaseline.create_from_report!(baz_report, baz_report.time)

      Summarizer.compare_group_against_node(group, baz_report.node_id)

      foo_baseline = NodeBaseline.find_by_node_id(foo_report.node.id)
      foo_baseline.differences.find_by_resource_name('File[/etc/sudoers]').accept_change

      diff = Diff.group_against_node(group, baz_report.node, baz_report.time, 0, 5)

      diff.should == [
        [
          "File[/etc/hosts]", [
            {
              :properties => {
                :inspected => {
                  "group" => "1",
                  "mode" => "655",
                  "owner" => "0",
                },
                :baseline => [
                  {
                    "group" => "0",
                    "mode" => "644",
                    "owner" => "1",
                  },
                ],
              },
              :digest => "Dz11WOTDLc8YwzGN+3ts8w==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1,
            },
            {
              :properties => {
                :inspected => {
                  "owner" => "0",
                },
                :baseline => [
                  {
                    "owner" => "1",
                  },
                ],
              },
              :digest => "Pa1BjVFP4aRZxctihvTFjw==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1,
            },
          ],
        ],
        [
          "File[/etc/sudoers]", [
            {
              :properties => {
                :inspected => {
                  "owner" => "0",
                },
                :baseline => [
                  {
                    "owner" => "1",
                  },
                ],
              },
              :digest => "BdGCiBagW+SY+vZqKAvQdg==",
              :accepted => "multiple",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 2,
            },
          ],
        ],
      ]
    end

    it "should omit corresponding property values for resources absent from the baseline" do
      foo_report = Report.create_from_yaml(fixture 'foo')
      bar_report = Report.create_from_yaml(fixture 'bar')
      baseline_report = Factory(:inspect_report, :time => foo_report.time)

      group = NodeGroup.generate!
      group.nodes = [foo_report.node, bar_report.node, baseline_report.node]
      group.save!

      baseline = NodeBaseline.create_from_report!(baseline_report, baseline_report.time)

      Summarizer.compare_group_against_node(group, baseline_report.node_id)

      diff = Diff.group_against_node(group, baseline_report.node, baseline_report.time, 0, 5)

      diff.should == [
        [
          "File[/etc/hosts]",
          [
            {
              :properties => {
                :inspected => { "group" => "1", "mode" => "655", "owner" => "0" },
                :baseline => []
              },
              :digest => "Dz11WOTDLc8YwzGN+3ts8w==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1
            },
            {
              :properties => {
                :inspected => { "group" => "0", "mode" => "644", "owner" => "0" },
                :baseline => []
              },
              :digest => "Pa1BjVFP4aRZxctihvTFjw==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1
            }
          ]
        ],
        [
          "File[/etc/sudoers]",
          [
            {
              :properties => {
                :inspected => { "group" => "0", "mode" => "440", "owner" => "0" },
                :baseline => []
              },
              :digest => "BdGCiBagW+SY+vZqKAvQdg==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 2
            }
          ]
        ],
        [
          "User[root]",
          [
            {
              :properties => {
                :inspected => { "shell" => "/bin/sh", "home" => "/root" },
                :baseline => []
              },
              :digest => "0F4Kaeb6aW6gwIVfcz+zGA==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1
            },
            {
              :properties => {
                :inspected => { "shell" => "/bin/sh", "home" => "/var/root" },
                :baseline => []
              },
              :digest => "ny1vSG/0drWD9R1773TbNA==",
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 1,
              :node_count => 1
            }
          ]
        ]
      ]
    end
  end

  describe ".group_against_own_baselines" do
    it "should diff every member of a group against its own baselines" do
      foo_report = Factory(:inspect_report, :host => "foo", :time => Time.zone.now)
      bar_report = Factory(:inspect_report, :host => "bar", :time => Time.zone.now)
      baz_report = Factory(:inspect_report, :host => "baz", :time => Time.zone.now)

      foo_baseline = NodeBaseline.create_from_report!(Report.create_from_yaml(fixture 'foo'), foo_report.time)
      bar_baseline = NodeBaseline.create_from_report!(Report.create_from_yaml(fixture 'bar'), bar_report.time)
      baz_baseline = NodeBaseline.create_from_report!(Report.create_from_yaml(fixture 'baz'), baz_report.time)

      group = NodeGroup.generate!
      group.nodes = [foo_report.node, bar_report.node, baz_report.node]
      group.save!

      Summarizer.queue_diff_summarization

      diff = Diff.group_against_own_baselines(group, Time.zone.now, 0, 5)

      diff.should == [
        [
          "File[/etc/hosts]",
          [
            {
              :digest => nil,
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 3,
              :properties => {
                :baseline => [
                  { "group" => "1", "mode" => "655", "owner" => "0" },
                  { "group" => "0", "mode" => "644", "owner" => "0" },
                  { "group" => "0", "mode" => "644", "owner" => "1" }
                ],
                :inspected => {}
              },
              :node_count => 3
            }
          ]
        ],
        [
          "File[/etc/sudoers]",
          [
            {
              :digest => nil,
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 2,
              :properties => {
                :baseline => [
                  { "group" => "0", "mode" => "440", "owner" => "0" },
                  { "group" => "0", "mode" => "440", "owner" => "1" }
                ],
                :inspected => {}
              },
              :node_count => 3
            }
          ]
        ],
        [
         "User[root]",
          [
            {
              :digest => nil,
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 2,
              :properties => {
                :baseline => [
                  { "shell" => "/bin/sh", "home" => "/root" },
                  { "shell" => "/bin/sh", "home" => "/var/root" }
                ],
                :inspected => {}
              },
              :node_count => 3
            }
          ]
        ]
      ]
    end
  end

  describe ".group_diff" do
    it "should return an empty hash when an empty list of node_baselines is provided" do
      Diff.group_diff([], 0, 57).should == []
    end

    it "should return an empty hash when no node_baselines are provided" do
      Diff.group_diff(nil, 0, 57).should == []
    end

    it "should filter results by resource" do
      Summarizer.stubs(:delay).returns(Summarizer)

      foo_report = Factory(:inspect_report, :host => "foo", :time => Time.zone.now)
      bar_report = Factory(:inspect_report, :host => "bar", :time => Time.zone.now)
      baz_report = Factory(:inspect_report, :host => "baz", :time => Time.zone.now)

      foo_baseline = NodeBaseline.create_from_report!(Report.create_from_yaml(fixture 'foo'), foo_report.time)
      bar_baseline = NodeBaseline.create_from_report!(Report.create_from_yaml(fixture 'bar'), bar_report.time)
      baz_baseline = NodeBaseline.create_from_report!(Report.create_from_yaml(fixture 'baz'), baz_report.time)

      group = NodeGroup.generate!
      group.nodes = [foo_report.node, bar_report.node, baz_report.node]
      group.save!

      Summarizer.queue_diff_summarization

      diff = Diff.group_diff([foo_baseline, bar_baseline, baz_baseline], 0, 100, 'file', 'sudoers')
      diff.should == [
        [
          "File[/etc/sudoers]",
          [
            {
              :digest => nil,
              :accepted => "no-action",
              :can_change => true,
              :baseline_count => 2,
              :properties => {
                :baseline => [
                  { "group" => "0", "mode" => "440", "owner" => "0" },
                  { "group" => "0", "mode" => "440", "owner" => "1" }
                ],
                :inspected => {}
              },
              :node_count => 3
            }
          ]
        ]
      ]
    end

    it "should return unique properties (and differences) on both sides" do
      nb = {}

      now  = Time.now
      4.times do |n|
        init = Factory(:inspect_report, :host => "data#{n}.local", :time => now - 1.day)

        # Add the content, by way of resources and their events, that we need.
        status = Factory(:resource_status, :report => init, :title => '/example')
        Factory(:resource_event,
                :resource_status => status,
                :property        => "ensure",
                :previous_value  => :file)

        Factory(:resource_event,
                :resource_status => status,
                :property        => "unchanged",
                :previous_value  => :unchanged)

        Factory(:resource_event,
                :resource_status => status,
                :property        => "different",
                :previous_value  => :"different_#{n%2}")

        Factory(:resource_event,
                :resource_status => status,
                :property        => "origin",
                :previous_value  => :origin)

        Factory(:resource_event,
                :resource_status => status,
                :property        => "baseline",
                :previous_value  => :baseline)

        initial_nb = NodeBaseline.create_from_report!(init, init.time)

        node_baseline = initial_nb.create_for_next_day
        report = Factory(:inspect_report, :host => "data#{n}.local", :time => now)

        status = Factory(:resource_status, :report => report, :title => '/example')
        Factory(:resource_event,
                :resource_status => status,
                :property        => "ensure",
                :previous_value  => :directory)

        Factory(:resource_event,
                :resource_status => status,
                :property        => "unchanged",
                :previous_value  => :unchanged)

        Factory(:resource_event,
                :resource_status => status,
                :property        => "different",
                :previous_value  => :outcome)

        Factory(:resource_event,
                :resource_status => status,
                :property        => "origin",
                :previous_value  => :"outcome_#{n%2}")

        Factory(:resource_event,
                :resource_status => status,
                :property        => "inspect",
                :previous_value  => :inspect)

        nb[node_baseline] = report
      end

      # Build all the differences...
      Summarizer.stubs(:delay).returns(Summarizer)
      Summarizer.queue_diff_summarization

      diff = Diff.group_diff(nb.keys, 0, 100)

      diff.length.should == 1
      diff.first.should be_an_instance_of Array

      diff = diff.first
      diff.length.should == 2
      diff.first.should  == "File[/example]"

      diff = diff.second
      diff.length.should == 2
      diff.map do |this|
        this.should be_an_instance_of Hash
        this[:node_count].should == 2
        this[:can_change].should be_true
        this[:baseline_count].should == 1
      end

      properties = diff.map do |this|
        this[:properties]
      end.group_by do |this|
        this[:inspected]["origin"]
      end

      properties.keys.should =~ [:outcome_0, :outcome_1]

      properties[:outcome_0].length.should == 1
      properties[:outcome_0].first.should == {
        :baseline => [{ "different" => :different_0,
                        "baseline"  => :baseline,
                        "origin"    => :origin,
                        "ensure"    => :file
                      }],
        :inspected => {
          "different" => :outcome,
          "origin"    => :outcome_0,
          "ensure"    => :directory,
          "inspect"   => :inspect
        }
      }

      properties[:outcome_1].length.should == 1
      properties[:outcome_1].first.should == {
        :baseline => [{ "different" => :different_1,
                        "baseline"  => :baseline,
                        "origin"    => :origin,
                        "ensure"    => :file}],
        :inspected => {
          "different" => :outcome,
          "origin"    => :outcome_1,
          "ensure"    => :directory,
          "inspect"   => :inspect
        }
      }
    end
  end
end
